package bia.rnn;

import bia.common.Utils;
import bia.sga.ContinuousFunction;
import bia.sga.SGA;

/**
 * Created by drchajan on 17/03/14.
 */
public class LearnToOscillate implements ContinuousFunction {
    private static final int SEQUENCE_LENGTH = 10;

    private RNN rnn;

    public LearnToOscillate() {
        this.rnn = new RNN(1, 2, 1);
    }

    public double[] targetSequence() {
        double[] s = new double[SEQUENCE_LENGTH];
        for (int i = 0; i < SEQUENCE_LENGTH; i++) {
//            s[i] = i % 2;
//            s[i] = (i % 3)/2.0;
            s[i] = 0.5 + Math.sin(i)/2.0;
        }
        return s;
    }

    public double evaluateForWeights(double[] w) {
        double[] s = targetSequence();
        rnn.setWeights(w);
        rnn.resetActivities();
        rnn.loadInputs(new double[]{1.0});

        double err = 0.0;
        for (int i = 0; i < SEQUENCE_LENGTH; i++) {
            rnn.propagate();//push data into the network
            double y = rnn.getOutputs()[0];//get the output
            double d = s[i];//desired output
            double e = y - d;
            err += e * e;
        }
        return err;
    }

    public double[] evaluateOutputsForWeights(double[] w) {
        double[] o = new double[2*SEQUENCE_LENGTH];

        rnn.setWeights(w);
        rnn.resetActivities();
        rnn.loadInputs(new double[]{1.0});

        for (int i = 0; i < 2*SEQUENCE_LENGTH; i++) {
            rnn.propagate();//push data into the network
            o[i] = rnn.getOutputs()[0];//get the output
        }
        return o;
    }

    @Override
    public double f(double[] x) {
        
        return -evaluateForWeights(x);
    }

    @Override
    public double[] o(int[] x) {
        
        return new double[1];
    }
    
    @Override
    public int getDimension() {
        return rnn.getNumWeights();
    }

    @Override
    public void store(double[] x) {
    }

    public static void main(String[] args) throws Exception {
        double[] best = SGA.run();  //--- find the best weight vector
        LearnToOscillate learnToOscillate = new LearnToOscillate();

        System.out.println("RNN error: " + learnToOscillate.evaluateForWeights(best));
        System.out.println("RNN weights: " + Utils.vectorToString(best));
        System.out.println("Target sequence : (" + Utils.vectorToString(learnToOscillate.targetSequence()) + ")");
        System.out.println("RNN response:     (" + Utils.vectorToString(learnToOscillate.evaluateOutputsForWeights(best)) + ")");
    }
}
